#!/bin/sh
#
# Copyright (c) 2002 Lubomir Sedlacik <salo@Xtrmntr.org>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.

# Tested on NetBSD, Solaris and GNU/Linux

#
# set some reasonable defaults
#
INSTALLDIR="/usr/lib/ispell"
CAT_PATH="/bin /usr/bin "
MKDIR_PATH="/bin /usr/bin"
RM_PATH="/bin /usr/bin /usr/xpg4/bin"
CP_PATH="/bin /usr/bin /usr/xpg4/bin"
SORT_PATH="/bin /usr/bin /usr/xpg4/bin"
INSTALL_PATH="/bin /usr/bin /usr/pkg/bin /usr/local/bin /usr/ccs/bin /usr/ucb /opt/sfw/bin /usr/sbin"
BUILDHASH_PATH="/bin /usr/bin /usr/pkg/bin /usr/local/bin /opt/sfw/bin"

#
# functions
#

print_intro() {
	echo ""
	echo "    >>>  Vita vas instalacny skript ispell-sk  <<<"
	echo ""
	echo ""
	echo " Instalacia prebieha v niekolkych krokoch: "
	echo ""
	echo " 1) instalacny skript si skontroluje, ci ma k dispozicii vsetky"
	echo "    potrebne programy k spravnemu fungovaniu"
	echo ""
	echo " 2) zvolite si adresar, kam sa ma slovnik nainstalovat."
	echo "    k dispozicii je predvoleny adresar [$INSTALLDIR]"
	echo ""
	echo " 3) vytvori sa slovnik v hash tvare a nainstaluje sa"
	echo "    do zvoleneho adresara"
	echo ""
	echo " 4) zmazu sa docasne subory"
	echo ""
	echo " Instalaciu mozete kedykolvek ukoncit stlacenim Ctrl-c"
	echo ""
}

print_error() {
	echo "ERROR: $1"
}

print_msg() {
	echo ">>> $1"
}

ask() {
	if [ `echo -n x` = "-n x" ]; then
		echo "$1"\\c
	else
		echo -n "$1"
	fi
}

check_tool() {
	TOOL="$1"
	POSSIBLE_DIRS="$2"
	VAR="$3"
	FOUND=0
	for DIR in $POSSIBLE_DIRS ; do
		if [ -x $DIR/$TOOL ]; then
			eval "$VAR"="$DIR/$TOOL" && FOUND=1
			break
		fi
	done

	if [ $FOUND -eq 0 ]; then
		print_error "Nebolo mozne najst $TOOL v ziadnom z adresarov $POSSIBLE_DIRS."
		exit 1
	fi
}

read_ano_nie() {
	MSG="$1"
	VAR="$2"

	while TRUE=1 ; do
		ask "$MSG (A/n): "
		read ANSWER
		case "x$ANSWER" in
			xa|xA|xano|xANO|x)
				eval "$VAR"=ano
				return
				;;
			xn|xN|xnie|xNIE)
				eval "$VAR"=nie
				return
				;;
			*)
				print_error "Neplatna odpoved [$ANSWER]"
				continue
				;;
		esac
	done
}

choose_installdir() {
	while TRUE=1 ; do
		ask "Zadajte cestu alebo stlacte <enter> pre [$INSTALLDIR]: "

		read NEWDIR
		if [ x$NEWDIR = x ]; then
			NEWDIR=$INSTALLDIR
		fi

		if [ -e $NEWDIR ]; then
			if [ -d $NEWDIR ]; then
				if [ -w $NEWDIR ]; then
					INSTALLDIR=$NEWDIR
					return
				else
					print_error "Do adresara $NEWDIR nie je mozne zapisat."
				fi
			else
				print_error "$NEWDIR nie je adresarom."
			fi
		else
			read_ano_nie "Adresar $NEWDIR neexistuje. Chcete ho vytvorit?" MAKEDIR
			if [ $MAKEDIR = ano ]; then
				${MKDIR} -p $NEWDIR 2> /dev/null
				if [ $? -ne 0 ]; then
					print_error "Adresar $NEWDIR nie je mozne vytvorit."
				else
					INSTALLDIR=$NEWDIR
					return
				fi
			fi
		fi
	done
}

do_install() {
	print_msg "Spajam a zotriedujem casti slovnika."
	${CAT} *.words | ${SORT} -u -f > slovak.words
        if [ $? -ne 0 ]; then
                print_error "Nepodarilo sa spojit a zotriedit casti slovnika."
        else
                print_msg "OK"
        fi

	print_msg "Kopirujem pravidla do docasneho suboru."
	${CP} slovak.aff slovak.aff.bak
        if [ $? -ne 0 ]; then
                print_error "Nepodarilo sa skopirovat pravidla do docasneho suboru."
        else    
                print_msg "OK"
        fi

	print_msg "Vytvaram hash slovnika."
	${BUILDHASH} slovak.words slovak.aff slovak.hash 2>/dev/null >/dev/null
        if [ $? -ne 0 ]; then
                print_error "Nepodarilo sa vytvorit hash slovnika."
        else    
                print_msg "OK"
        fi

	print_msg "Instalujem slovnik v hash forme."
	${INSTALL} -m 0644 slovak.hash ${INSTALLDIR}/slovak.hash
        if [ $? -ne 0 ]; then
                print_error "Nepodarilo sa nainstalovat slovnik v hash forme."
        else
                print_msg "OK"
	fi

	print_msg "Instalujem pravidla."
	${INSTALL} -m 0644 slovak.aff.bak ${INSTALLDIR}/slovak.aff
        if [ $? -ne 0 ]; then
                print_error "Nepodarilo sa nainstalovat pravidla."
        else
                print_msg "OK"
	fi
}

cleanup() {
	${RM} -f slovak.words slovak.words.cnt slovak.words.stat slovak.hash slovak.aff.bak *~
	if [ $? -ne 0 ]; then
		print_error "Nepodarilo sa zmazat vsetky docasne subory."
	else
		print_msg "OK"
	fi	
}

#
# main
#

print_intro
print_msg "Stlacte <enter> pre pokracovanie..."
read CONTINUE

print_msg "1) Hladam potrebne nastroje..."
check_tool cat "$CAT_PATH" CAT
check_tool rm "$RM_PATH" RM
check_tool cp "$CP_PATH" CP
check_tool mkdir "$MKDIR_PATH" MKDIR
check_tool sort "$SORT_PATH" SORT
check_tool install "$INSTALL_PATH" INSTALL
check_tool buildhash "$BUILDHASH_PATH" BUILDHASH
print_msg "OK"
echo ""

print_msg "2) Kam chcete nainstalovat slovnik?"
choose_installdir
print_msg "OK"
echo ""

print_msg "3) Instalujem slovnik..."
do_install
echo ""

print_msg "4) Mazem docasne subory..."
cleanup
echo ""

print_msg "Hotovo."
